"""Script for running a tsunami inundation scenario for Southwest coast,
Sri Lanka.

Source data such as elevation and boundary data is assumed to be available in
directories specified by project.py
The output sww file is stored in directory named after the scenario, i.e
slide or fixed_wave.

The scenario is defined by a triangular mesh created from project.polygon,
the elevation data and a tsunami wave generated by a submarine mass failure.

Ole Nielsen and Duncan Gray, GA - 2005 and Jane Sexton and
Nick Bartzis, GA - 2006
"""

#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------

# Standard modules
import os
import time
import sys
from os.path import dirname, basename

# Related major packages
from anuga.shallow_water import Domain
from anuga.shallow_water import Reflective_boundary
from anuga.shallow_water import Dirichlet_boundary
from anuga.shallow_water import Time_boundary
from anuga.shallow_water import File_boundary
from anuga.shallow_water import Field_boundary

from anuga.pmesh.mesh_interface import create_mesh_from_regions
from anuga.shallow_water.data_manager import convert_dem_from_ascii2netcdf
from anuga.abstract_2d_finite_volumes.util import start_screen_catcher, copy_code_files
from anuga.shallow_water.data_manager import dem2pts
from anuga.geospatial_data.geospatial_data import *
from os import sep
from anuga.utilities.polygon import read_polygon, Polygon_function

# Application specific imports
import project                 # Definition of file names and polygons

copy_code_files(project.output_run_time_dir,__file__, 
                dirname(project.__file__)+sep+ project.__name__+'.py' )

start_screen_catcher(project.output_run_time_dir)

#------------------------------------------------------------------------------
# Create the triangular mesh based on overall clipping polygon with a tagged
# boundary and interior regions defined in project.py along with
# resolutions (maximal area of per triangle) for each polygon
#------------------------------------------------------------------------------

create_mesh_from_regions(project.bounding_polygon,
                         boundary_tags={'back': [3,4,5,6], 'side': [0,2],
                                            'ocean': [1]},
                         maximum_triangle_area=project.regional_res,
                         filename=project.meshes_dir_name+'.msh',
                         interior_regions=project.interior_regions,
                         use_cache=False,
                         verbose=True)


#------------------------------------------------------------------------------
# Setup computational domain
#------------------------------------------------------------------------------

domain = Domain(project.meshes_dir_name+'.msh', use_cache=False, verbose=True)

print domain.statistics()

domain.set_name(project.scenario_name)
domain.set_datadir(project.output_run_time_dir)
domain.set_default_order(2) # Apply second order scheme
domain.set_minimum_storable_height(0.01) # Don't store anything less than 1cm
domain.set_store_vertices_uniquely(False)
domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
domain.set_maximum_allowed_speed(0.1) # Allow a little runoff (0.1 is OK)
domain.beta_h = 0 # Use first order near bathymetry (faster)

#------------------------------------------------------------------------------
# Setup initial conditions
#------------------------------------------------------------------------------


IC = Polygon_function( [(project.poly_mainland, -1.0)], default = project.tide,
                         geo_reference = domain.geo_reference)
domain.set_quantity('stage', IC)
domain.set_quantity('friction', 0.01) 
print'start set elevation quantity'
domain.set_quantity('elevation', 
#                    filename=project.combined_dem_name + '.txt',
                    filename=project.combined_canal_fort_dem_name + '.txt',
#                    filename=project.combined_canal_dem_name + '.txt',
#                    filename=project.combined_dem_name + '.pts',
                    use_cache=True,
                    verbose=True,
                    alpha=0.1)
print'finish set elevation quantity'
#------------------------------------------------------------------------------
# Setup boundary conditions
#------------------------------------------------------------------------------

#print 'Available boundary tags', domain.get_boundary_tags()

Br = Reflective_boundary(domain)
Bd = Dirichlet_boundary([project.tide,0,0])

Bf = Field_boundary(project.boundaries_dir_name+'.sww',
                    domain, time_thinning=2, mean_stage=project.tide, 
                    use_cache=True, verbose=True)

# 60 min square wave starting at 1 min, 50m high
#if scenario == 'fixed_wave':

Bw = Time_boundary(domain = domain,
                       f=lambda t: [(60<t<300)*3, 0, 0])
                       
domain.set_boundary({'back': Br,
                         'side': Bd,
#                         'ocean': Bw}
                         'ocean': Bf}
                         )

    
#------------------------------------------------------------------------------
# Evolve system through time
#------------------------------------------------------------------------------

t0 = time.time()

from anuga.abstract_2d_finite_volumes.quantity import Quantity

for t in domain.evolve(yieldstep = 60, finaltime = 30000):
    domain.write_time()
    domain.write_boundary_statistics(tags = 'ocean')
            
print 'That took %.2f seconds' %(time.time()-t0)
